import torch
import os
import poisoned_datasets
import poisoning_adversary
from tqdm import tqdm
import torchvision
from robustness import data_augmentation as da, model_utils
from argparse import ArgumentParser 
parser = ArgumentParser() 
parser.add_argument('--mask_path', type=str, required=True) 
parser.add_argument('--eps', type=float, required=True) 
args = parser.parse_args() 
ds = poisoned_datasets.CIFAR(os.path.join(os.environ['SHARDATA'], 'datasets/cifar'))

model, checkpoint_loaded = model_utils.make_and_restore_model(
    arch='resnet18',
    dataset=ds,
    resume_path='/storage/anon/cifar/clean/84ee88cb-7daa-40f0-8d58-2d91900bc63f/checkpoint.pt.latest'
)

train_set = torchvision.datasets.CIFAR10(
    root='/storage/anon/datasets/cifar', 
    train=True,
    transform=da.TEST_TRANSFORMS_DEFAULT(32)
)
loader = torch.utils.data.DataLoader(train_set, batch_size=64, shuffle=False)

already_vulnerable_mask_train = []

import math
iterations = math.ceil(1.5 * args.eps / 0.00025)

for batchx, batchy in tqdm(loader):
    output = model(
        batchx.cuda(), batchy.cuda(),
        make_adv=True, constraint='inf',
        eps=args.eps, random_restarts=10,
        # iterations=20,
        iterations=iterations,
        step_size=0.00025
    )
    out_mask = torch.argmax(output[0], dim=-1) == batchy.cuda()
    already_vulnerable_mask_train.extend(out_mask.detach().cpu().numpy())

x, y = poisoning_adversary.unzip(train_set)

split_mask = [[] for i in range(10)]
for m, y_ in zip(already_vulnerable_mask_train, y):
    split_mask[y_].append(m)

import pickle
print('saving mask to {}'.format(args.mask_path))
pickle.dump(
    split_mask,
    open(args.mask_path, 'wb')
)

