import attack
import matplotlib.pyplot as plt
from tqdm import tqdm
import os
import numpy as np
from data_loader import *
import tensorflow as tf
from copy import deepcopy
import sys

random_seed = int(sys.argv[1])

tf.random.set_seed(random_seed)
np.random.seed(random_seed)


def unzip(v):
    x = list(map(lambda x: x[0], v))
    y = list(map(lambda x: x[1], v))
    return (x, y)

# divides data points into 2 classes: dominated and dominating
def get_dominating_dataset(x, flip_mask):
    dominating = filter(lambda i: i[1], zip(x, flip_mask))
    non_dominating = filter(lambda i: not i[1], zip(x, flip_mask))
    dominating = unzip(dominating)[0]
    non_dominating = unzip(non_dominating)[0]
    return (dominating, non_dominating)

def get_labels(dataset):
    labels = []
    for i, class_points in enumerate(dataset):
        labels.append([i for point in class_points])
    return labels

def size_split(size, split_ratio):
    first = int(size * split_ratio)
    second = size - first
    return (first, second)


# input must be in our dataset format - different lists for different classes
def split_into_train_test(x, train_ratio):
    train_data = []
    test_data = []
    for class_points in x:
        np.random.shuffle(class_points)
        train_size, test_size = size_split(len(class_points), split_ratio=train_ratio)
        train_data.append(class_points[: train_size])
        test_data.append(class_points[train_size: ])
    
    return train_data, test_data




def get_dominating_non_dominating_dataset(dataset, classes, flip_budget, gammas, adversary, norm):
    dataset, test_dataset = load_dataset(dataset=dataset, classes=classes)
    # eta values we are interested in 
    record_checkpoints = [dict((i, {'taken':None, 'dominated':None}) for i in [flip_budget]) for j in range(classes)]
    # call first adversary to poison dataset
    for class_points, rc, gamma in zip(dataset, record_checkpoints, gammas):
        adversary.get_dominating_set(points=class_points, norm=norm, norm_threshold=gamma, record_checkpoints=rc)
    # get flip and dominated masks
    flip = [record_checkpoint[flip_budget]['taken'] for record_checkpoint in record_checkpoints]
    dataset = [get_dominating_dataset(*x_flip) for x_flip in zip(dataset, flip)]
    dominating, not_dominating = unzip(dataset)
    for non_dominating_class_points in not_dominating:
        np.random.shuffle(non_dominating_class_points)
    for i in range(len(not_dominating)):
        not_dominating[i] = not_dominating[i][: flip_budget]
    dominating = np.concatenate(dominating)
    not_dominating = np.concatenate(not_dominating)

    x = [dominating, not_dominating]

    return split_into_train_test(x, train_ratio=0.7)


def get_dominating_set_classifier(architecture, dataset, classes, adversary, flip_budget, norm, gammas, train_epochs, batch_size):

    train_data, test_data = get_dominating_non_dominating_dataset(dataset=dataset, classes=classes, flip_budget=flip_budget, gammas=gammas, adversary=adversary, norm=norm)

    model = attack.model.get_fit_net(
                            name=architecture,
                            units=2,
                            src=None,
                            train_data_x=np.concatenate(train_data),
                            train_data_labels=np.concatenate(get_labels(train_data)),
                            test_data_x=np.concatenate(test_data),
                            test_data_labels=np.concatenate(get_labels(test_data)),
                            epochs=train_epochs,
                            batch_size=batch_size
                        )
    return model


# make changes here
get_dominating_set_classifier(architecture='cnn', dataset='mnist', classes=10, adversary=attack.first_adversary.Naive(), flip_budget=64, norm=np.inf, gammas=[0.9, 0.52, 0.9883, 0.98, 0.96, 0.98, 0.9, 0.88, 0.975, 0.9], train_epochs=300, batch_size=128)
